@@ -84,7 +84,7 @@ As a record object (record data and user data gotten from object),
 e.g.,
 
     $record->data( $record_data );
-    $recore->user( $user_data );
+    $record->user( $user_data );
     $record = $dshash{''} = $record;
 
 Note that in the last example, the object fetched is not the same as
@@ -108,11 +108,11 @@ saves you from having to do something like this equivalent code:
  
 =head1 VERSION
 
-FlatFile::DataStore::Tiehash version 1.02
+FlatFile::DataStore::Tiehash version 1.03
 
 =cut
 
-our $VERSION = '1.02';
+our $VERSION = '1.03';
 
 use 5.008003;
 use strict;
